@@ -93,7 +93,11 @@ ldns_sign_public(ldns_rr_list *rrset, ldns_key_list *keys)
 			printf("[XX]ERROR NO SIGN BUG, OUT OF MEM?\n");
 			ldns_rr_list_print(stdout, rrset_clone);
 			fflush(stdout);
-			exit(123);
+			ldns_rr_list_free(rrset_clone);
+			ldns_rr_list_free(signatures);
+			ldns_rdf_free(first_label);
+			ldns_rdf_free(new_owner);
+			return NULL;
 		}
 		b64rdf = NULL;
 
@@ -339,8 +343,9 @@ ldns_sign_public_evp(ldns_buffer *to_sign,
 	/* initializes a signing context */
 	md_type = digest_type;
 	if(!md_type) {
-		printf("Unknown message digest");
-		exit(1);
+		/* unknown message difest */
+		ldns_buffer_free(b64sig);
+		return NULL;
 	}
 
 	EVP_MD_CTX_init(&ctx);
@@ -627,22 +632,27 @@ ldns_dnssec_zone_create_nsec3s(ldns_dnssec_zone *zone,
 	current_name_node = first_name_node;
 
 	while (current_name_node &&
-		  current_name_node != LDNS_RBTREE_NULL) {
+	       current_name_node != LDNS_RBTREE_NULL) {
 		current_name = (ldns_dnssec_name *) current_name_node->data;
 		nsec_rr = ldns_dnssec_create_nsec3(current_name,
-									NULL,
-								     zone->soa->name,
-									algorithm,
-									flags,
-									iterations,
-									salt_length,
-									salt);
+		                                   NULL,
+		                                   zone->soa->name,
+		                                   algorithm,
+		                                   flags,
+		                                   iterations,
+		                                   salt_length,
+		                                   salt);
+		/* by default, our nsec based generator adds rrsigs
+		 * remove the bitmap for empty nonterminals */
+		if (!current_name->rrsets) {
+			ldns_rdf_deep_free(ldns_rr_pop_rdf(nsec_rr));
+		}
 		ldns_rr_set_ttl(nsec_rr, nsec_ttl);
 		ldns_dnssec_name_add_rr(current_name, nsec_rr);
 		ldns_rr_list_push_rr(new_rrs, nsec_rr);
 		ldns_rr_list_push_rr(nsec3_list, nsec_rr);
 		current_name_node = ldns_dnssec_name_node_next_nonglue(
-						    ldns_rbtree_next(current_name_node));
+		                   ldns_rbtree_next(current_name_node));
 	}
 
 	ldns_rr_list_sort_nsec3(nsec3_list);
@@ -939,6 +949,9 @@ ldns_dnssec_zone_sign_nsec3(ldns_dnssec_zone *zone,
 									 iterations,
 									 salt_length,
 									 salt);
+				/* always set bit 7 of the flags to zero, according to
+				 * rfc5155 section 11 */
+				ldns_set_bit(ldns_rdf_data(ldns_rr_rdf(nsec3params, 1)), 7, 0);
 				ldns_dnssec_zone_add_rr(zone, nsec3params);
 				ldns_rr_list_push_rr(new_rrs, nsec3params);
 			}
